@@ -1,4 +1,6 @@
 import logging, json, os
+from fastmcp.server.auth.providers.jwt import StaticTokenVerifier
+from fastmcp.server.auth.providers.jwt import JWTVerifier
 from fastmcp import FastMCP
 from fastmcp.prompts.prompt import PromptMessage, TextContent
 from agentmake import agentmake, DEVELOPER_MODE, readTextFile
@@ -13,7 +15,24 @@ AGENTMAKE_CONFIG["backend"] = config.backend
 # Configure logging before creating the FastMCP server
 logging.basicConfig(format="[%(levelname)s]: %(message)s", level=logging.ERROR)
 
-mcp = FastMCP(name="BibleMate AI")
+BIBLEMATE_STATIC_TOKEN = os.getenv("BIBLEMATE_STATIC_TOKEN")
+BIBLEMATE_MCP_PUBLIC_KEY = os.getenv("BIBLEMATE_MCP_PUBLIC_KEY")
+
+verifier = StaticTokenVerifier(
+    tokens={
+        BIBLEMATE_STATIC_TOKEN: {
+            "client_id": "biblemate-ai",
+            "scopes": ["read:data", "write:data", "admin:users"]
+        },
+    },
+    required_scopes=["read:data"]
+) if BIBLEMATE_STATIC_TOKEN else JWTVerifier(
+    public_key=BIBLEMATE_MCP_PUBLIC_KEY,
+    issuer=os.getenv("BIBLEMATE_MCP_ISSUER"),
+    audience=os.getenv("BIBLEMATE_MCP_AUDIENCE")
+) if BIBLEMATE_MCP_PUBLIC_KEY else None
+
+mcp = FastMCP(name="BibleMate AI", auth=verifier)
 
 def getResponse(messages:list) -> str:
     return messages[-1].get("content") if messages and "content" in messages[-1] else "Error!"
