@@ -6,13 +6,15 @@ from biblemate import config, AGENTMAKE_CONFIG, BIBLEMATEDATA, fix_string, write
 from biblemate.uba.api import run_uba_api, DEFAULT_MODULES
 from pathlib import Path
 import urllib.parse
-import asyncio, re, os, subprocess, click, gdown, pprint, argparse, json, zipfile
+import asyncio, re, os, subprocess, click, gdown, pprint, argparse, json, zipfile, shutil
 from copy import deepcopy
 from alive_progress import alive_bar
 from fastmcp import Client
+from fastmcp.client.transports import StreamableHttpTransport
 from agentmake.plugins.uba.lib.BibleBooks import BibleBooks
 from agentmake import agentmake, getOpenCommand, getDictionaryOutput, edit_file, edit_configurations, readTextFile, writeTextFile, getCurrentDateTime, AGENTMAKE_USER_DIR, USER_OS, DEVELOPER_MODE, DEFAULT_AI_BACKEND
 from agentmake.utils.handle_text import set_log_file_max_lines
+from agentmake.utils.manage_package import getPackageLatestVersion
 from rich.console import Console
 from rich.markdown import Markdown
 from rich.progress import Progress, SpinnerColumn, TextColumn
@@ -30,7 +32,16 @@ set_log_file_max_lines(log_file, config.max_log_lines)
 # AI backend
 version = readTextFile(os.path.join(os.path.dirname(os.path.realpath(__file__)), "version.txt"))
 parser = argparse.ArgumentParser(description = f"""BibleMate AI {version} CLI options""")
+# global options
+parser.add_argument("default", nargs="*", default=None, help="initial prompt")
 parser.add_argument("-b", "--backend", action="store", dest="backend", help="AI backend; overrides the default backend temporarily.")
+parser.add_argument("-l", "--lite", action="store", dest="lite", choices=["true", "false"], help="Enable / disable lite context. Must be one of: true, false.")
+parser.add_argument("-m", "--mode", action="store", dest="mode", choices=["agent", "partner", "chat"], help="Specify AI mode. Must be one of: agent, partner, chat.")
+parser.add_argument("-pe", "--promptengineer", action="store", dest="promptengineer", choices=["true", "false"], help="Enable / disable prompt engineering. Must be one of: true, false.")
+parser.add_argument("-s", "--steps", action="store", dest="steps", type=int, help="Specify the maximum number of steps allowed.")
+parser.add_argument("-e", "--exit", action="store_true", dest="exit", help="exit after the first response (for single-turn use cases).")
+# mcp options
+parser.add_argument("-t", "--token", action="store", dest="token", help="specify a static token to use for authentication with the MCP server; applicable to command `biblemate` only")
 parser.add_argument("-mcp", "--mcp", action="store", dest="mcp", help=f"specify a custom MCP server to use, e.g. 'http://127.0.0.1:{config.mcp_port}/mcp/'; applicable to command `biblemate` only")
 parser.add_argument("-p", "--port", action="store", dest="port", help=f"specify a port for the MCP server to use, e.g. {config.mcp_port}; applicable to command `biblematemcp` only")
 args = parser.parse_args()
@@ -43,13 +54,34 @@ else:
     with open(os.path.join(os.path.dirname(os.path.realpath(__file__)), "config.py"), "a", encoding="utf-8") as fileObj:
         fileObj.write(f'''\nbackend="{DEFAULT_AI_BACKEND}"''')
     config.backend = DEFAULT_AI_BACKEND
+
 AGENTMAKE_CONFIG["backend"] = config.backend
+DEFAULT_SYSTEM = "You are BibleMate AI, an autonomous agent designed to assist users with their Bible study."
+DEFAULT_MESSAGES = [{"role": "system", "content": DEFAULT_SYSTEM}, {"role": "user", "content": "Hello!"}, {"role": "assistant", "content": "Hello! I'm BibleMate AI, your personal assistant for Bible study. How can I help you today?"}] # set a tone for bible study; it is userful when auto system is used.
+
+# other temporary config changes
+if args.lite == "true":
+    config.lite = True
+elif args.lite == "false":
+    config.lite = False
+if args.mode == "agent":
+    config.agent_mode = True
+elif args.mode == "partner":
+    config.agent_mode = False
+elif args.mode == "chat":
+    config.agent_mode = None
+if args.promptengineer == "true":
+    config.prompt_engineering = True
+elif args.promptengineer == "false":
+    config.prompt_engineering = False
+if args.steps:
+    config.max_steps = args.steps
 
 def mcp():
     builtin_mcp_server = os.path.join(os.path.dirname(os.path.realpath(__file__)), "bible_study_mcp.py")
     user_mcp_server = os.path.join(AGENTMAKE_USER_DIR, "biblemate", "bible_study_mcp.py") # The user path has the same basename as the built-in one; users may copy the built-in server settings to this location for customization. 
     mcp_script = readTextFile(user_mcp_server if os.path.isfile(user_mcp_server) else builtin_mcp_server)
-    mcp_script = mcp_script.replace("mcp.run(show_banner=False)", f'''mcp.run(show_banner=False, transport="http", port={args.port if args.port else config.mcp_port})''')
+    mcp_script = mcp_script.replace("mcp.run(show_banner=False)", f'''mcp.run(show_banner=False, transport="http", host="0.0.0.0", port={args.port if args.port else config.mcp_port})''')
     exec(mcp_script)
 
 def main():
@@ -125,45 +157,60 @@ Get a static text-based response directly from a text-based AI model without usi
     
     return tools, tools_schema, master_available_tools, available_tools, tool_descriptions, prompts, prompts_schema, resources, templates
 
+def edit_temp_file(initial_content: str) -> str:
+    temp_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), "temp", "edit.md")
+    writeTextFile(temp_file, initial_content)
+    edit_file(temp_file)
+    return readTextFile(temp_file).strip()
+
 def backup_conversation(messages, master_plan, console=None):
     """Backs up the current conversation to the user's directory."""
-    # determine storage path
-    if console:
-        timestamp = getCurrentDateTime()
-        storagePath = os.path.join(AGENTMAKE_USER_DIR, "biblemate", "chats", timestamp)
-    else:
-        storagePath = os.path.join(os.path.dirname(os.path.realpath(__file__)), "temp")
-    # create directory if not exists
-    if not os.path.isdir(storagePath):
-        Path(storagePath).mkdir(parents=True, exist_ok=True)
-    # Save full conversation
-    conversation_file = os.path.join(storagePath, "conversation.py")
-    writeTextFile(conversation_file, pprint.pformat(messages))
-    # Save master plan
-    writeTextFile(os.path.join(storagePath, "master_plan.md"), master_plan)
-    # Save markdown
-    markdown_file = os.path.join(storagePath, "conversation.md")
-    markdown_text = "\n\n".join(["```"+i["role"]+"\n"+i["content"]+"\n```" for i in messages if i.get("role", "") in ("user", "assistant")])
-    writeTextFile(markdown_file, markdown_text)
-    # Save html
-    if console:
-        html_file = os.path.join(storagePath, "conversation.html")
-        console.save_html(html_file, inline_styles=True, theme=MONOKAI)
-    # Inform users of the backup location
-    if console:
-        print(f"Conversation backup saved to {storagePath}")
-        print(f"Report saved to {html_file}\n")
+    if len(messages) > len(DEFAULT_MESSAGES):
+        # determine storage path
+        if console:
+            timestamp = getCurrentDateTime()
+            storagePath = os.path.join(AGENTMAKE_USER_DIR, "biblemate", "chats", timestamp)
+        else:
+            storagePath = os.path.join(os.path.dirname(os.path.realpath(__file__)), "temp")
+        # create directory if not exists
+        if not os.path.isdir(storagePath):
+            Path(storagePath).mkdir(parents=True, exist_ok=True)
+        # Save full conversation
+        conversation_file = os.path.join(storagePath, "conversation.py")
+        writeTextFile(conversation_file, pprint.pformat(messages))
+        # Save master plan
+        writeTextFile(os.path.join(storagePath, "master_plan.md"), master_plan)
+        # Save markdown
+        markdown_file = os.path.join(storagePath, "conversation.md")
+        markdown_text = "\n\n".join(["```"+i["role"]+"\n"+i["content"]+"\n```" for i in messages if i.get("role", "") in ("user", "assistant")])
+        writeTextFile(markdown_file, markdown_text)
+        # Save html
+        if console:
+            html_file = os.path.join(storagePath, "conversation.html")
+            console.save_html(html_file, inline_styles=True, theme=MONOKAI)
+        # Inform users of the backup location
+        if console:
+            print(f"Conversation backup saved to {storagePath}")
+            print(f"Report saved to {html_file}\n")
 
 async def main_async():
 
+    BIBLEMATE_STATIC_TOKEN = args.token if args.token else os.getenv("BIBLEMATE_STATIC_TOKEN")
+    BIBLEMATE_MCP_PRIVATE_KEY=os.getenv("BIBLEMATE_MCP_PRIVATE_KEY")
+
     # The client that interacts with the Bible Study MCP server
     if args.mcp:
         mcp_server = f"http://127.0.0.1:{config.mcp_port}/mcp/" if args.mcp == "biblemate" else args.mcp
-        client = Client(mcp_server)
+        transport = StreamableHttpTransport(
+            mcp_server,
+            auth=BIBLEMATE_STATIC_TOKEN if BIBLEMATE_STATIC_TOKEN else BIBLEMATE_MCP_PRIVATE_KEY if BIBLEMATE_MCP_PRIVATE_KEY else None,
+        )
+        client = Client(transport=transport)
     else:
         builtin_mcp_server = os.path.join(os.path.dirname(os.path.realpath(__file__)), "bible_study_mcp.py")
         user_mcp_server = os.path.join(AGENTMAKE_USER_DIR, "biblemate", "bible_study_mcp.py") # The user path has the same basename as the built-in one; users may copy the built-in server settings to this location for customization. 
-        client = Client(user_mcp_server if os.path.isfile(user_mcp_server) else builtin_mcp_server)
+        mcp_server = user_mcp_server if os.path.isfile(user_mcp_server) else builtin_mcp_server        
+        client = Client(mcp_server) # no auth for local server
 
     APP_START = True
     DEFAULT_SYSTEM = "You are BibleMate AI, an autonomous agent designed to assist users with their Bible study."
@@ -177,7 +224,7 @@ async def main_async():
     async with client:
         tools, tools_schema, master_available_tools, available_tools, tool_descriptions, prompts, prompts_schema, resources, templates = await initialize_app(client)
         resource_suggestions = json.loads(run_uba_api(".resources"))
-        resource_suggestions = resource_suggestions["bibleListAbb"]+resource_suggestions["commentaryListAbb"]+resource_suggestions["encyclopediaListAbb"]+resource_suggestions["lexiconList"]
+        resource_suggestions = [f"//bible/{i}/" for i in resource_suggestions["bibleListAbb"]]+[f"//commentary/{i}/" for i in resource_suggestions["commentaryListAbb"]]+[f"//encyclopedia/{i}/" for i in resource_suggestions["encyclopediaListAbb"]]+[f"//lexicon/{i}/" for i in resource_suggestions["lexiconList"]]
         abbr = BibleBooks.abbrev["eng"]
         resource_suggestions += [abbr[str(book)][0] for book in range(1,67)]
 
@@ -250,6 +297,23 @@ async def main_async():
                         console.print("Restart to make the changes in the backend effective!", justify="center")
                         console.rule()
                         exit()
+                # check for updates
+                latest_version = getPackageLatestVersion("biblemate")
+                current_version = readTextFile(os.path.join(os.path.dirname(os.path.realpath(__file__)), "version.txt")).strip()
+                if latest_version and latest_version.strip() != current_version:
+                    console.rule()
+                    if USER_OS == "Windows":
+                        console.print(Markdown(f"## A new version of BibleMate AI is available: {latest_version} (you are using {current_version}).\n\nPlease run `pip install --upgrade biblemate` to upgrade!"))
+                        console.rule()
+                    elif click.confirm(f"A new version of BibleMate AI is available: {latest_version} (you are using {current_version}). Do you want to upgrade now?", default=True) and shutil.which("pip"):
+                        pip_path = shutil.which("pip")
+                        if os.system(f"{pip_path} install --upgrade biblemate") == 0:
+                            console.print("Restart to make the changes effective!", justify="center")
+                            console.rule()
+                            exit()
+                        else:
+                            console.print(Markdown(f"## A new version of BibleMate AI is available: {latest_version} (you are using {current_version}).\n\nPlease run `pip install --upgrade biblemate` to upgrade!"))
+                            console.rule()
             # Original user request
             # note: `python3 -m rich.emoji` for checking emoji
             console.print("Enter your request :smiley: :" if len(messages) == len(DEFAULT_MESSAGES) else "Enter a follow-up request :flexed_biceps: :")
@@ -273,7 +337,13 @@ async def main_async():
                 ".help": "help page",
             }
             input_suggestions = list(action_list.keys())+["@ ", "@@ "]+[f"@{t} " for t in available_tools]+[f"{p} " for p in prompt_list]+[f"//{r}" for r in resources.keys()]+template_list+resource_suggestions # "" is for generating ideas
-            user_request = await getInput("> ", input_suggestions)
+            if args.default:
+                user_request = " ".join(args.default)
+                args.default = None # reset to avoid repeated use
+            else:
+                user_request = await getInput("> ", input_suggestions)
+            if user_request == ".editprompt": # edit current prompt in editor
+                user_request = edit_temp_file(config.current_prompt)
             while not user_request.strip():
                 # Generate ideas for `prompts to try`
                 ideas = ""
@@ -287,8 +357,10 @@ async def main_async():
                 console.rule()
                 console.print(Markdown(f"## Ideas\n\n{ideas}\n\n"))
                 console.rule()
-                # Get input agin
+                # Get input again
                 user_request = await getInput("> ", input_suggestions)
+                if user_request == ".editprompt": # edit current prompt in editor
+                    user_request = edit_temp_file(config.current_prompt)
 
             # display resources
             if user_request.startswith("//") and user_request[2:] in resources:
@@ -364,7 +436,7 @@ async def main_async():
                     # load conversation
                     if os.path.isfile(load_path):
                         file_path = load_path
-                    elif os.path.isdir(load_path) and os.path.isfile(os.path.join(load_path, "conversation.py")) and os.isfile(os.path.join(load_path, "master_plan.md")):
+                    elif os.path.isdir(load_path) and os.path.isfile(os.path.join(load_path, "conversation.py")) and os.path.isfile(os.path.join(load_path, "master_plan.md")):
                         file_path = os.path.join(load_path, "conversation.py")
                     else:
                         print("Expected a file or a directory containing `conversation.py` and `master_plan.md`.")
@@ -625,8 +697,7 @@ async def main_async():
                         tool_properties = tool_schema["parameters"]["properties"]
                         if len(tool_properties) == 1 and "request" in tool_properties: # AgentMake MCP Servers or alike
                             if "items" in tool_properties["request"]: # requires a dictionary instead of a string
-                                request_dict = [{"role": "system", "content": DEFAULT_SYSTEM}]+messages[len(messages)-2:] if config.lite else messages
-                                request_dict += [{"role": "user", "content": tool_instruction}]
+                                request_dict = [{"role": "system", "content": DEFAULT_SYSTEM}]+messages[len(messages)-2:] if config.lite else deepcopy(messages)
                                 tool_result = await client.call_tool(tool, {"request": request_dict})
                             else:
                                 tool_result = await client.call_tool(tool, {"request": tool_instruction})
@@ -705,6 +776,8 @@ Available tools are: {available_tools}.
                         console.print("Please review and confirm the master plan, or make any changes you need:", justify="center")
                         console.rule()
                         master_plan_edit = await getInput(default_entry=master_plan)
+                        if master_plan_edit == ".editprompt": # edit current prompt in editor
+                            master_plan_edit = edit_temp_file(config.current_prompt)
                         if not master_plan_edit or master_plan_edit == ".quit":
                             if messages and messages[-1].get("role", "") == "user":
                                 messages = messages[:-1]
@@ -785,6 +858,8 @@ Available tools are: {available_tools}.
                     console.print("Please review and confirm the next step, or make any changes you need:")
                     console.rule()
                     next_step_edit = await getInput(default_entry=next_step)
+                    if next_step_edit == ".editprompt": # edit current prompt in editor
+                        next_step_edit = edit_temp_file(config.current_prompt)
                     if not next_step_edit or next_step_edit == ".quit":
                         console.rule()
                         console.print("I've stopped processing for you.")
@@ -841,5 +916,8 @@ Please provide me with the final answer to my original request based on the work
             print()
             backup_conversation(messages, master_plan, console)
 
+            if args.exit:
+                break
+
 if __name__ == "__main__":
     asyncio.run(main())
